<?xml version="1.0" encoding="UTF-8"?>
<gui name="rocs"
     version="11"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd" >

    <MenuBar>
        <Menu name="file" >
            <Action name="new-project" />
            <Action name="open-project" />
            <Action name="recent-project" />
            <Separator/>
            <Action name="save-project" />
            <Action name="save-project-as" />
<!--             <Action name="change-project" /> -->

            <Separator/>
<!--             <Separator/> -->
        </Menu>
        <!--<Menu name="project" >
            <text>Project</text>
            <Action name="set-project-name" />
            <Separator/>
            <Separator/>
        </Menu>-->
        <Menu name="script">
            <text>Script Document</text>
            <Action name="new-script" />
            <Action name="add-script" />
            <Action name="export-script" />
        </Menu>
        <Menu name="graph">
            <text>Graph Document</text>
            <Action name="new-graph" />
            <Action name="import-graph" />
            <Action name="export-graph-as" />
<!--             <Action name="save-graph" /> -->
            <!--<Menu name="graph_file">
                <text>Graph File</text>
            </Menu>-->
            <Separator/>
<!--            <Menu name="graph_selected">
                <text>Edit Selected Data</text>
                <Menu>
                    <text>Layout</text>
                    <Action name="align-htop"/>
                    <Action name="align-hcenter"/>
                    <Action name="align-hbottom"/>
                    <Action name="align-vleft"/>
                    <Action name="align-vcenter"/>
                    <Action name="align-vright" />
                    <Action name="align-circle" />
                    <Action name="align-tree" />
                </Menu>
                <Action name="delete-selected" />
            </Menu>-->
            <Menu>
                <text>Tools</text>
                <ActionList name="tools_plugins" />
            </Menu>
        </Menu>
    </MenuBar>
</gui>

